"""
WebDebug HTTP message

This is the superclass to both HTTPrequest and HTTPresponse.


WebDebug is a web development proxy server that captures and displays HTTP
requests and responses.
Copyright (C) 1999  Paul Clip.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""

### IMPORTS

from WDconst import *

import string
import time
import math
import urllib


### CONSTANTS

# used for pickling
RAWSIZE = "rawsize"
CREATION_TIME = "creation_time"
IP_ADDRESS = "ip_address"
LOAD_TIME = "load_time"
WDVERSION = "WDversion"
TOO_HIGH_TO_MEASURE = 0

### Classes

class HTTPmessage:
	"""
	The super class to HTTPrequest and HTTPresponse.
	Member variables:
		rawsize: the total size of the HTTP message in bytes, including headers
		creation_time: the time at which the message was instantiated
		ip_address: ip address & port of the remote host (browser for requests, server for responses)
		load_time: the time at which the message finished transferring (must be set by subclasses)
		WDversion: the WebDebug version number this class was saved as
	"""

	def __init__(self, ip):
		self.rawsize = 0
		self.creation_time = time.time()
		self.ip_address = ip
		self.load_time = -1
		self.WDversion = VERSION_NUMBER

	def readLine(self, input):
		""" Reads a line from input and keeps track of total size of message """
		str = input.readline()
		self.rawsize = self.rawsize + len(str)
		return str

	def getIPandPort(self):
		""" Returns the IP address and port of the connection """
		return "%s:%d" % (self.ip_address[0], self.ip_address[1])

	def getRawSize(self):
		""" Returns the raw size of the message in bytes """
		return self.rawsize

	def getThroughput(self):
		""" Returns the bytes/sec throughput to download the message """
		download_speed = self.getLoadTime() - self.getCreationTime()
		if download_speed == 0:
			return TOO_HIGH_TO_MEASURE
		else: 
			return self.getRawSize() / download_speed

	def getThroughputString(self):
		""" Returns the throughput to download the message, handles division by zero exceptions """
		throughput = self.getThroughput()
		if throughput != TOO_HIGH_TO_MEASURE:
			return "%d bytes/sec" % throughput
		else:
			return "Too high to measure"

	def getCreationTime(self):
		""" Returns the time at which the message was created """
		return self.creation_time

	def getLoadTime(self):
		""" Returns the time at which the message finished loading """
		if self.load_time == -1:
			raise "HTTPmessage.getLoadTime(): load time not set by subclass before use"
		return self.load_time

	def findHeaderValue(self, headers, key, toreturn = None):
		""" returns the value associated with the key passed in (lowercased), returns None """
		#print "Key is %s" % key
		#print "Headers are %s" % key
		#print "Toreturn is %s" % toreturn
		lckey = string.lower(key)
		for i in headers.keys():
			if stripRand(i) == lckey:
				return headers[i]
		return toreturn

	def parseHeaders(self, input):
		""" Parses HTTP message headers and returns a sequence of [Key, Value] pairs """
		headers = {}
		raw_headers = {}
		line = self.readLine(input)
		while ':' in line:
			i = string.find(line, ':')
			raw_headers[addRand(line[:i])] = string.strip(line[i+1:])
			headers[addRand(string.lower(line[:i]))] = string.strip(line[i+1:])
			line = input.readline()
		return raw_headers, headers

	def readContents(self, input, read_anyway=0):
		""" Reads the body of an HTTP message """
		raw_contents = ""
		#print "self.headers is %s" % self.headers
		content_length = self.findHeaderValue(self.headers, 'content-length')
		if content_length != None:
			raw_contents = input.read(int(content_length))
		elif read_anyway:
			# BUGBUG this may not read everything for very slow responses
			raw_contents = input.read()
		self.rawsize = self.rawsize + len(raw_contents)
		return raw_contents

	def findNameValue(self, str):
		""" Return a url decoded name=value pair in the string str """
		equal = string.find(str, '=')
		if equal == -1:
			name = str
			value = ""
		else:
			name = urllib.unquote_plus(str[:equal])
			value = urllib.unquote_plus(str[equal+1:])
		return name, value
			
	def printError(self, errorstr):
		""" Prints an error message to the console """
		print "HTTPmessage(%s) error: %s" % (id(self), errorstr)

	def stripTrailer(self, str):
		""" Strips the tailing \\n or \\r\\n from a HTTP header line """
		requestline = str
		if requestline[-2:] == '\r\n':
			requestline = requestline[:-2]
		elif requestline[-1:] == '\n':
			requestline = requestline[:-1]
		return requestline

	def shortTime(self, t):
		""" Return an HH:MM:SS.MS string based on time t """
		t1 = time.strftime("%H:%M:%S", time.localtime(t))
		return "%s.%s" % (t1, ("%.3f" % math.modf(t)[0])[2:])

	def unEscape(self, str):
		""" Returns an unescaped version of str, i.e. %xx are converted to their proper char """
		res = ""
		i = 0
		while i < len(str):
			if str[i] != '%':
				res += str[i]
				i += 1
			else:
				res += chr(int(str[i+1:i+3], 16))
				i += 3
		return res
	
	def toHTML(self, commandline):
		""" Renders the common parts of the HTTP messages to HTML """
		html = "<table border=1 width=800><th align=left colspan=2 bgcolor=#dddddd>" + commandline + "</th>\n"
		for h in self.raw_headers.keys():
			key = stripRand(h)
			val = self.raw_headers[h]
#			if key == "Cookie":
#				val = self.unEscape(val)
#				val = self.unEscape(val)
			html = html + "<tr><td bgcolor=#dddddd>%s:</td><td>%s</td>\n" % (key, val)
		html = html + "<tr><td>Size of message</td><td>%s bytes</td>\n" % self.getRawSize()
		html = html + "<tr><td>Message received at</td><td>%s</td>\n" % self.shortTime(self.getLoadTime())
		return html

